# RBD INSTRUMENTS 9103 SAMPLE APPLICTION
# 9103 Standard Speed Sampling

# Copyright (C) 2021 RBD Instruments, Inc.

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    https://www.gnu.org/licenses/



# MODULES USED
#pyserial

import serial


#SETUP SERIAL port
#standard speed

#Query user for port number
port_number = input("Enter a port number: ")
print(port_number)

#setup port
try:
    port=serial.Serial(
        'com' + port_number,
        baudrate=57600,
        bytesize=serial.EIGHTBITS,
        parity=serial.PARITY_NONE,
        stopbits=serial.STOPBITS_ONE,
        xonxoff=False,
        timeout=1)
except:
    print('9103 not found at ' + port_number + '\n Press Ctrl-C to quit')
    try:
        while True:
            pass
    except KeyboardInterrupt:
            pass



# BASIC COMMANDS
# These are defaults for startup (no args)
def command_range_auto():
    port.write(message('R0')) # put in autorange

def command_default_filter_32():
    port.write(message('F032'))

def command_default_input_normal():
    port.write(message('G0'))

def command_default_bias_off():
    port.write(message('B0'))


#MESSAGE PARSING HELPERS

#Convert string to bytes for message to 9103_port
#all messages must end with newline
def message(message_string):
    return bytes('&'+ message_string + '\n','utf-8')

#Single sample (standard speed) message parsing
def parse_message_for_sample( msg):
    if '&S' in msg:
        return msg.strip('&')
    else:
        return ''


#INITIALIZE 9103

command_range_auto()
command_default_filter_32()
command_default_input_normal()
command_default_bias_off()



# SAMPLING LOOP
# with keyboard interrupt (ctrl c)

print('Press Ctrl-C to stop sampling')
port.write(message('I1000'))

#write to file
import os

desktop_path = os.path.join(os.path.join(os.environ['USERPROFILE']), 'Desktop')

#samplefile= open('C:\\Users\\joeca\Desktop\\test.txt', 'w')
samplefile= open(desktop_path + '\\9103-sample-log.txt', 'w')


try:
    while True:
        msg=port.readline().decode('utf-8').rstrip()
        print(msg)
        parsed = parse_message_for_sample(msg)
        if parsed:
            samplefile.write( parsed + '\n')

except KeyboardInterrupt:
    pass

samplefile.close()
port.write(message("I0000"))
print("Sampling stopped")

print("Press Ctrl-C to quit")

try:
    while True:
        pass

except KeyboardInterrupt:
    pass
