# RBD INSTRUMENTS 9103 SAMPLE APPLICTION
# 9103 Switch Speed Mode Utility

# Copyright (C) 2021 RBD Instruments, Inc.

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    https://www.gnu.org/licenses/



#   The 9103 with the high-speed option recalls the last speed mode it was operated at.
#   This makes it easier to program applications normally only operate in one speed mode.
#   However, because the different modes open the serial ports at different baud rates
#   (57.6k for standard, 230.4k for high) it may not be known what mode the 9103 was last operating in,
#   so an application could fail if it assumes the incorrect speed.
#
#   Since you can't query the 9103 speed mode withouth opening the port,
#   one way to get around the issue is to catch a port failure, and try a different baud rate.
#
#   This utiity tries to open the port for standard speed, and if that fails, high speed.
#   In either case, if the port is succesfully opened and communications with the 9103 established,
#   the user is prompted to switch the speed mode.
#
#   NOTE: The failure conditions / timeouts may be different on different operating systems.
#   and need to be modified.
#   This version works on Windows 10.


# MODULES USED
#pyserial
import serial

import time


# BASIC COMMANDS
# These are defaults for startup (no args)

def command_query_status():
    port.write(message('Q'))

def command_switch_to_standard_speed():
    port.write(message('US'))

def command_switch_to_high_speed():
    port.write(message('UF'))


#MESSAGE PARSING HELPERS

#Convert string to bytes for message to 9103_port
#all messages must end with newline
def message(message_string):
    return bytes('&'+ message_string + '\n','utf-8')



#SETUP SERIAL PORT

#Query user for port number
port_number = input("Enter a port number: ")
print(port_number)

#setup port
port_open = True
high_speed = False



#TRY OPENING AND R/W IN STANDARD SPEED
#note that this if there is a 9103 on the selected port, writing will not necessarily throw
#an exception - we need to try and read to confirm we are talking at the selected speed

try:
    port=serial.Serial(
        'com' + port_number,
        baudrate=57600,
        bytesize=serial.EIGHTBITS,
        parity=serial.PARITY_NONE,
        stopbits=serial.STOPBITS_ONE,
        xonxoff=False,
        timeout=0)
except:
    port_open = False
    do_nothing = input('9103 standard speed not found. Press enter to continue...')

if (port_open == True):
    do_nothing = input('9103 port opened at standard speed. Press enter to continue...')

standard_speed = False;
timeout = 3   # seconds
timeout_start = time.time()
try:
    command_query_status()

    while time.time() < timeout_start + timeout:
        msg=port.readline().decode('utf-8').rstrip()

        if msg == "RBD Instruments: PicoAmmeter":
            print(msg)
            standard_speed = True;

except Exception as e:
     print(e)
     do_nothing = input('Could not read at standard speed. Press enter to continue...')
     port.close()

     #TRY OPENING AND R/W IN HIGH SPEED
     try:
         port=serial.Serial(
             'com' + port_number,
             baudrate=230400,
             bytesize=serial.EIGHTBITS,
             parity=serial.PARITY_NONE,
             stopbits=serial.STOPBITS_ONE,
             xonxoff=False,
             timeout=0)
     except:
         port_open = False
         do_nothing = input('9103 high speed not found. Press enter to continue...')

     if (port_open == True):
          do_nothing = input('9103 port opened at high speed. Press enter to continue...')

     timeout_start = time.time()
     try:
         command_query_status()
         while time.time() < timeout_start + timeout:
             msg=port.readline().decode('utf-8').rstrip()
             if msg == "RBD Instruments: PicoAmmeter":
                 print(msg)
                 standard_speed = False;

     except Exception as e:
        print(e)
        do_nothing = input('Could not read at high speed. Press enter to continue...')
        port.close()
        do_nothing = input('Exiting. Press enter to continue...')
        exit()


#SUCCESS - prompt user to switch from standard to high or high to stanrard
if (standard_speed):
    switch = input('9103 port opened at standard speed. Switch to High Speed (y/n)?')
    if (switch=='y'):
        command_switch_to_high_speed()
        do_nothing = input('Switched to high speed. Press enter to continue...')
else:
    switch = input('9103 port opened at high speed. Switch to Standard Speed (y/n)?')
    if (switch=='y'):
        command_switch_to_standard_speed()
        do_nothing = input('Switched to standard speed. Press enter to continue...')

#EXIT
do_nothing = input('Exiting. Press enter to continue...')
exit()
